/***************************************************************************//**
 *   @file   ADN2850.c
 *   @brief  Implementation of ADN2850 Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 559
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "ADN2850.h"			// ADN2850 definitions.
#include "Communication.h"		// Communication definitions.

/**************************************************************************//**
 * @brief Initialize SPI.
 *
 * @return 1.
******************************************************************************/
unsigned char ADN2850_Init(void)
{
    ADI_PART_CS_PIN_OUT;
    ADI_PART_CS_HIGH;    // Active LOW
    ADN2850_RESET_OUT;
    ADN2850_RESET_HIGH;  // Active LOW
    ADN2850_WP_OUT;
    ADN2850_WP_HIGH;     // Active LOW
    ADN2850_RDY_IN;  
    SPI_Init(0, 1000000, 1, 1);
  
    return(1);    
}

/**************************************************************************//**
 * @brief Send a command to the device. It will transmit 3 bytes.
 *
 * @param command - will be used to select the command that will be sent
 *                  Example: - ADN2850_WRITE_RDAC_CMD
 *                           - ADN2850_INCREMENT_1STEP_CMD, etc.
 * @param address - will be used to set the address section of a command
 *                  Example: - ADN2850_RDAC1
 *                           - ADN2850_USER4
 *                           - ADN2850_RWB1_TOLERANCE, etc.
 *                  
 * @param Data - will be used to set the data section of a command
 *               (valid values 0-65535).
 *
 * @return None
******************************************************************************/
void ADN2850_SendCommand(unsigned char command, 
                         unsigned char address, 
                         unsigned short Data)
{
    unsigned char spiWord[4] = {0, 0, 0, 0};
    unsigned char *data = (unsigned char*)&Data;
    
    spiWord[0] = 0x01;
    spiWord[1] = (command << 4) | (address & 0x0F);
    spiWord[2] = data[1];
    spiWord[3] = data[0];
    SPI_Write(spiWord, 3);
}

/**************************************************************************//**
 * @brief Read from register or from EEMEM.
 *
 * @param readType - Reads from register or from EEMEM
 *                   Example: - ADN2850_READ_RDAC_CMD - register read.
 *                            - ADN2850_READ_EEM_CMD  - memory read.
 *                  
 * @param address -  will be used to set the address section of a command.
 *                   Example: - ADN2850_RDAC1
 *                            - ADN2850_USER4
 *                            - ADN2850_RWB1_TOLERANCE, etc.
 *                  
 *
 * @return readData - data read from the device.
******************************************************************************/
unsigned short ADN2850_Read(unsigned char readType, unsigned char address)
{
    unsigned char readData[4] = {0x01, 0x00, 0x00, 0x00};
        
    ADN2850_SendCommand(readType, address, 0x00);
    SPI_Read(readData, 3);
    
    return(((unsigned short)readData[1] << 8) + readData[2]);
}

/**************************************************************************//**
 * @brief A subtle feature of the ADN2850 is that a subsequent CS strobe, 
 *        without clock and data, repeats a previous command.
 *
 * @return None.
******************************************************************************/
void ADN2850_RepeatLastCMD(void)
{
    ADI_PART_CS_LOW;
    ADI_PART_CS_HIGH;
}

/**************************************************************************//**
 * @brief Determines the programmed output resistance between Terminal Bx and 
 *        Terminal Wx.
 *
 * @return resistance - The output resistance valuein Ohms.
******************************************************************************/
unsigned short ADN2850_OhmConversion(unsigned long rawData)
{
    float resistance = 0;
    
    resistance = rawData * 24.4 + // The step corresponding to 1LSB is 24.4[Ohm]. 
                 30;              // Wiper resistance.    
    
    return((unsigned short) resistance);
}
